package com.jasamedika.medifirst2000.service;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.jasamedika.medifirst2000.entities.AntrianPasienDiPeriksa;
import com.jasamedika.medifirst2000.entities.RencanaOperasi;
import com.jasamedika.medifirst2000.vo.AntrianPasienDiPeriksaVO;
import com.jasamedika.medifirst2000.vo.RencanaOperasiCustomVO;
import com.jasamedika.medifirst2000.vo.RencanaOperasiVO;

public interface RencanaOperasiService {
	
	Map<String,Object> saveRencanaOperasi(RencanaOperasiVO vo);
	Map<String, Object> findByName(String nama);
	Map<String, Object> savePermintaanOperasi(RencanaOperasiCustomVO vo);
	Map<String, Object> saveRencanaOperasiNew(RencanaOperasiCustomVO vo);
	Map<String, Object> saveOperasi(RencanaOperasiVO vo);

	Map<String, Object> savePembatalanOperasi(RencanaOperasiVO vo);
	List<Map<String, Object>> getRencanaOperasi (String noRec);
	List<Map<String, Object>> getPermintaanOperasi ();
	List<Map<String, Object>> getRencanaOperasi ();	
	List<Map<String, Object>> getPermintaanOperasi (String startDate, String endDate, String noCm);
	List<Map<String, Object>> getRencanaOperasi (String startDate, String endDate, String noCm);	
	Map<String, Object> getRencanaOperasiByNorec(String noRec);
	Map<String, Object> getPermintaanOperasiByNorec(String noRec);
	
	
}
